#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'stringk'
__mtime__ = '2020/8/21'
# qq | WX:2456056533

佛祖保佑  永无bug!

"""

import random
import string
import aiohttp
import os
import time
import asyncio
import numpy as np
import cv2
from aiohttp import ClientTimeout
from aiohttp.client_exceptions import ServerTimeoutError

from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client

from clife_svc.errors.error_code import FileException, ParameterException, UploadImgException, TimeoutException
from clife_svc.libs.log import klogger


class ClientRequest:

    _COS_CLIENT = ''
    HTTP_TIMEOUT = None
    COS_REGION = None
    COS_SECRET_ID = None
    COS_SECRET_KEY = None
    COS_BUCKET = None
    COS_DIR = None
    COS_BUCKET_HOST = None
    # 请求超时时间
    __http_sess_timeout = None

    def __init__(self, conf_item: dict):
        self.HTTP_TIMEOUT = int(conf_item.get('http.timeout', 5))
        self.COS_REGION = conf_item.get('cos.region', '')
        self.COS_SECRET_ID = conf_item.get('cos.secret.id', '')
        self.COS_SECRET_KEY = conf_item.get('cos.secret.key', '')
        self.COS_BUCKET = conf_item.get('cos.bucket', '')
        self.COS_DIR = conf_item.get('cos.dir', 'ai')
        self.COS_BUCKET_HOST = conf_item.get('cos.bucket.host', 'cos.clife.net')
        self.__http_sess_timeout = ClientTimeout(total=self.HTTP_TIMEOUT)

    def create_txy_client(self) -> CosS3Client:
        '''
        腾讯云上传client对象
        '''
        try:
            config = CosConfig(Region=self.COS_REGION, Secret_id=self.COS_SECRET_ID,
                               Secret_key=self.COS_SECRET_KEY,
                               Token=None)
            client = CosS3Client(config)
            return client
        except Exception as e:
            raise UploadImgException(error_code=9951, msg='create_txy_client error:{}'.format(e))

    @staticmethod
    async def _request_get(session, url, params, resp_type):
        async with session.get(url, params=params) as resp:
            if resp.status == 200:
                if resp_type == 'json':
                    return await resp.json()
                elif resp_type == 'text':
                    return await resp.text()
                else:
                    return await resp.read()
            else:
                klogger.error(
                    'error of ClientRequest._request_get,resp.status:{},resp.text:{}'.format(resp.status,
                                                                                             await resp.text()))

    @staticmethod
    async def _request_post(session, url, data, resp_type):
        async with session.post(url, data=data) as resp:
            if resp.status == 200:
                if resp_type == 'json':
                    return await resp.json()
                elif resp_type == 'text':
                    return await resp.text()
                else:
                    return await resp.read()
            else:
                klogger.error(
                    'error of ClientRequest._request_post,resp.status:{},resp.text:{}'.format(resp.status,
                                                                                              await resp.text()))

    async def _async_request(self, method, url, params=None, data=None, headers=None, cookies=None, resp_type='json'):
        '''

        :param method:
        :param url:
        :param params:
        :param data:
        :param headers:
        :param cookies:
        :param resp_type: json | text | byte
        :return:
        '''
        try:
            async with aiohttp.ClientSession(headers=headers, cookies=cookies, timeout=self.__http_sess_timeout) as sess:
                if method == 'GET':
                    return await self._request_get(sess, url, params, resp_type=resp_type)
                elif method == 'POST':
                    return await self._request_post(sess, url, data, resp_type=resp_type)
                else:
                    raise ParameterException(error_code=9982, msg='async_request method must in [GET,POST]')

        except ServerTimeoutError as aiohttp_time_out:
            raise TimeoutException(error_code=9940, msg='aiohttp_time_out:{},url:{}'.format(aiohttp_time_out, url))

    async def download_img(self, img_url, retry=2):
        '''
        图片下载
        :param img_url:
        :param retry:
        :return: img_byte | str
        '''
        while retry > 0:
            retry -= 1
            if 'http' in img_url:
                klogger.info('start download img:{}'.format(img_url))
                start = time.time()
                resp_byte = await self._async_request('GET', img_url, resp_type='byte')
                if resp_byte:
                    klogger.info('download img cost:{}s'.format(round(time.time() - start, 2)))
                    klogger.info('success download img')
                    return resp_byte
            else:
                # 本地文件路径格式直接返回
                klogger.info('img path:{}'.format(img_url))
                return ''

    async def get_img_np(self, img_url):
        '''
        图片下载
        :param img_url:
        :return: img np.asarray
        '''
        if img_url:
            resp = await self.download_img(img_url)
            if resp:
                image_array = np.frombuffer(resp, dtype=np.uint8)
                image_np = cv2.imdecode(image_array, cv2.IMREAD_COLOR)
                return image_np
            else:
                image_np = cv2.imread(img_url)
                if image_np is None:
                    raise FileException(error_code=9971, msg='img_file not exist')
                return image_np

    @staticmethod
    async def _rename_file(file: str):
        '''文件更名'''
        salt = ''.join(random.sample(string.ascii_letters + string.digits, 8))
        salt += str(int(time.time()))
        return salt + '_' + file

    async def upload_img(self, img_path: str, retry=2):
        '''
        上传图片至腾讯云cos
        :param img_path 待上传的图片路径
        :param retry 失败重试次数
        :return
        '''

        while retry > 0:
            retry -= 1
            start = time.time()
            if not self._COS_CLIENT:
                self._COS_CLIENT = self.create_txy_client()

            if not os.path.isfile(img_path):
                raise ParameterException(error_code=9987, msg='img not exist:'.format(img_path))

            file_name = os.path.split(img_path)[1]
            file_name = await self._rename_file(file_name)

            cloud_path = '/' + self.COS_DIR + '/' + file_name

            client_resp = self._COS_CLIENT.upload_file(
                Bucket=self.COS_BUCKET,
                LocalFilePath=img_path,
                Key=cloud_path,
                PartSize=10,
                MAXThread=10
            )
            '''
            client_resp:dict
            {'Content-Length': '0', 'Connection': 'keep-alive', 'Date': 'Thu, 27 Aug 2020 07:19:34 GMT', 
            'ETag': '"d2110b267778c3fd81854ff0283c01d4"', 'Server': 'tencent-cos', 'x-cos-hash-crc64ecma': '3062988045414607577', 
            'x-cos-request-id': 'NWY0NzVlODFfNThhYTk0MGFfNDQ1OV8zOTFkODUz'}

            '''
            if client_resp and isinstance(client_resp, dict):
                etag = str(client_resp.get('ETag', ''))
                if etag:
                    img_url = 'http://' + self.COS_BUCKET_HOST + cloud_path
                    klogger.info('upload img cost:{}s'.format(round(time.time() - start, 2)))
                    klogger.info('upload img success:{}'.format(img_url))
                    return img_url
            else:
                klogger.warning('error upload img,retry:{}'.format(retry))
                continue

        raise UploadImgException(error_code=9952, msg='client.put_object error')

    @staticmethod
    def case_get_img_np(img_url):
        '''测试从腾讯云下载图片'''
        loop = asyncio.get_event_loop()
        image_np = loop.run_until_complete(ClientRequest.get_img_np(img_url))
        print(image_np)

    @staticmethod
    def case_upload_img(img_url):
        loop = asyncio.get_event_loop()
        a = loop.run_until_complete(ClientRequest.upload_img(img_url))
        print(a)


if __name__ == '__main__':
    ClientRequest.case_get_img_np(img_url='http://cos.clife.net/10101/3cb34041583e4adebe241e8612d67eb2.png')
    # ClientRequest.case_upload_img(img_url=r'F:\star\hand.jpg')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'stringk'
__mtime__ = '2020/8/25'
# qq | WX:2456056533

佛祖保佑  永无bug!

"""
from typing import Optional, Dict, Any

from fastapi import HTTPException, Request
from fastapi.responses import JSONResponse


# class UnicornException(Exception):
#     def __init__(self, error_code=500, data={}, msg=''):
#         self.error_code = error_code
#         self.msg = msg
#         self.data = data
#
#
# async def unic_exception_handler(request: Request, exc: UnicornException):
#     return JSONResponse(
#         content={
#             "error_code": exc.error_code,
#             "msg": exc.msg,
#             "data": exc.data,
#         },
#         status_code=400,
#     )
#

class ApiException(HTTPException):
    status_code: int = 500  # 响应状态码
    error_code: int = 10000  # 错误状态码
    data: dict = {}
    msg: str = 'server error'
    headers: Optional[Dict[str, Any]] = None

    def __init__(self, status_code=None, error_code=None, data=None,
                 msg=None, headers=None):
        if status_code:
            self.status_code = status_code
        if error_code:
            self.error_code = error_code
        if data:
            self.data = data
        if msg:
            self.msg = msg

        if headers:
            self.headers = headers

        super(ApiException, self).__init__(
            status_code=self.status_code, detail=self.msg, headers=self.headers
        )

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r},error_code={self.error_code!r}, msg={self.msg!r}),data={self.data!r}"


async def api_exception_handler(_: Request, exc: ApiException) -> JSONResponse:
    return JSONResponse({
        "error_code": exc.error_code,
        "msg": exc.msg,
        "data": exc.data
    }, status_code=exc.status_code)


class ParameterException(ApiException):
    '''
    参数异常
    error_code 9980~9989
    '''
    status_code = 400
    error_code = 9980
    msg = 'invalid parameter'


class FileException(ApiException):
    '''
    文件异常
    error_code 9970~9979
    '''
    status_code = 400
    error_code = 9970
    msg = 'file error'


class TokenException(ApiException):
    '''
    Token异常
    error_code 9960~9969
    '''
    status_code = 400
    error_code = 9960
    msg = 'token error'


class UploadImgException(ApiException):
    '''
    文件上传异常
    error_code 9950~9959
    '''
    status_code = 400
    error_code = 9950
    msg = 'upload error'


class TimeoutException(ApiException):
    '''
    超时异常
    error_code 9940~9949
    '''
    status_code = 400
    error_code = 9940
    msg = 'timeout'


class AlgException(ApiException):
    '''
    模型加载/数据预处理/模型预测异常
    error_code 8000~8999
    '''
    status_code = 400
    error_code = 8000
    msg = 'alg exception'


if __name__ == '__main__':
    per = ParameterException(msg='缺少img_url参数')
    print(per.status_code)
    print(per.error_code)
    print(per.msg)

    ap = ApiException()
    print(ap.status_code)
    print(ap.error_code)
    print(ap.msg)



import asyncio
from typing import Optional

import uvicorn

from build.lib.clife_svc.errors.error_code import ParameterException
from clife_svc.application import App

app = App('clife_svc_test', '/www/logs')


async def detect(client_params: Optional[dict] = None) -> str:
    res = {'code': 0, 'msg': 'success'}
    # image_np = await app.get_img_np('http://cos.clife.net/10101/3cb34041583e4adebe241e8612d67eb2.png')
    # print(image_np)
    # res1 = await app.download_img('http://skintest.hetyj.com/ai/2R0FUj3S1630052673_28444.png')
    # print(res1)
    # await asyncio.sleep(3)
    # raise ParameterException()
    url = await app.upload_img('D:\\abc.xlsx')
    print(url)
    data = await app.download_img(url)
    print(data)
    return res

if __name__ == '__main__':
    app.add_api('/detect', detect, methods=['GET'])
    print(app.get_all_conf())
    loop = asyncio.get_event_loop()
    get_future = asyncio.ensure_future(app.download_img('http://skintest.hetyj.com/ai/WHbtvzo91630315253_abc.xlsx'))
    loop.run_until_complete(get_future)
    print(get_future.result())
    uvicorn.run(app.init_api(), host='0.0.0.0', port=30000, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
import time
from typing import (
    Any,
    Callable,
    List,
    Optional,
    Set,
    Union,
)

from fastapi import APIRouter
from fastapi import FastAPI
from fastapi import Request
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import JSONResponse
from fastapi.responses import ORJSONResponse
from starlette.middleware.base import (BaseHTTPMiddleware, RequestResponseEndpoint)
from starlette.requests import Request as HttpRequest
from starlette.responses import Response as HttpResponse

from clife_svc.disconf import Disconf
from clife_svc.errors.error_code import ApiException, api_exception_handler
from clife_svc.libs.http_request import ClientRequest
from clife_svc.libs.log import init_conf_log
from clife_svc.libs.log import init_svc_log
from clife_svc.libs.log import klogger


class App(object):
    '''
    http接口服务上下文对象，单实例对象
    '''
    __instance = None
    __conf_item = None
    __fast_api = FastAPI(title='ai-service', default_response_class=ORJSONResponse)
    __ai_router = APIRouter()
    __ClientRequest = None
    __app_name = None
    __log_path = None

    def __new__(cls, app_name: str, log_root_path: str, *conf: str):
        '''
        构造函数
        :param app_name 项目名称
        :param log_root_path 项目输出的日志根路径
        :param conf: 配置文件名称列表
        '''
        if not cls.__instance:
            cls.__instance = super().__new__(cls)
            cls.__log_path = log_root_path + '/' + app_name + '/'
            init_conf_log(cls.__log_path)
            cls.__conf_item = Disconf('clife-ai', '0.0.1-SNAPSHOT', conf).get_config_dict()
            cls.__ClientRequest = ClientRequest(cls.__conf_item)
            cls.__app_name = app_name
            init_svc_log(cls.__log_path, log_level='DEBUG')
        return cls.__instance

    def init_api(self) ->FastAPI:
        '''
        在App中初始化服务接口
        :return: FastAPI，作为服务器运行入口对象
        '''
        self.__fast_api.add_exception_handler(ApiException, api_exception_handler)
        init_middlewares(self.__fast_api)
        self.__ai_router.add_api_route('/time', index, methods=['GET'])
        self.__fast_api.include_router(self.__ai_router)
        self.__fast_api.add_exception_handler(Exception, app_exception)
        return self.__fast_api

    def get_conf(self, key: str):
        '''
        获取配置数据
        :param key:
        :return:
        '''
        try:
            return self.__conf_item[key]
        except KeyError:
            klogger.warning('config key not exist:{}'.format(key))
            return ''

    def get_all_conf(self) ->dict:
        '''
        获取所有配置数据
        :return:
        '''
        return self.__conf_item

    def add_api(self, path: str, endpoint: Callable[..., Any], methods: Optional[Union[Set[str], List[str]]] = None):
        '''
        增加服务接口，此函数需要在init_api前调用
        :param path:接口访问路径
        :param endpoint:接口实现函数
        :param methods:接口访问方式，如GET、POST等
        :return:
        '''
        self.__ai_router.add_api_route(path, endpoint, methods=methods)

    async def download_img(self, img_url, retry=2):
        '''
        下载图片
        :param img_url:图片地址
        :param retry:失败重试次数
        :return:图片数据字节数组
        '''
        return await self.__ClientRequest.download_img(img_url, retry)

    async def get_img_np(self, img_url):
        '''
        下载图片
        :param img_url:图片地址
        :return:图片数据np数组
        '''

        '''
        cos_cli = self.__ClientRequest.create_txy_client()
        buckets_list = cos_cli.list_buckets()
        for bucket in buckets_list['Buckets']['Bucket']:
            print(bucket)
            # acl = cos_cli.get_bucket_acl(bucket['Name'])
            # print(acl)
        # acl = cos_cli.get_bucket_acl('clife-cos-1251053011')
        # print(acl)
        '''
        return await self.__ClientRequest.get_img_np(img_url)

    async def upload_img(self, img_path: str, retry=2):
        '''

        :param img_path:
        :param retry:
        :return:
        '''
        return await self.__ClientRequest.upload_img(img_path, retry)


class Interceptor(BaseHTTPMiddleware):
    '''

    '''
    async def dispatch(self, request: HttpRequest, call_next: RequestResponseEndpoint) -> HttpResponse:
        klogger.info('Request url:{}'.format(request.url))
        klogger.info('Request query_params:{}'.format(request.query_params))
        start = time.time()
        response = await call_next(request)
        cost_time = time.time() - start
        klogger.info('Request cost:{}s'.format(round(cost_time, 2)))
        return response


def init_middlewares(app: FastAPI):
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )
    app.add_middleware(Interceptor)


async def index(q: Optional[str] = None):
    '''k8s 探针 http监控服务请求地址'''
    if q:
        return {'error_code': 0, 'msg': 'success',
                'data': {'time': time.strftime('%Y-%m-%d-%H-%M', time.localtime()), 'q': q}}

    return {'error_code': 0, 'msg': 'success', 'data': {'time': time.strftime('%Y-%m-%d-%H-%M', time.localtime())}}


async def app_exception(request: Request, exc: Exception):
    '''拦截所有未知 非HTTPException 异常'''
    klogger.info('Request url:{}'.format(request.url))
    klogger.info('Request path_params:{}'.format(request.path_params))
    klogger.info('Request query_params:{}'.format(request.query_params))
    klogger.info('Request exception:{}'.format(exc))

    return JSONResponse({
        "error_code": 99999,
        "msg": 'sorry for error:{}'.format(exc),
        "data": {},
    }, status_code=500)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'stringk'
__mtime__ = '2020/12/28'
# qq | WX:2456056533

佛祖保佑  永无bug!

"""
import os
from qcloud_cos import CosConfig, CosS3Client

from clife_svc.libs.http_request import ClientRequest

from clife_svc.libs.log import klogger


class TxyCos:

    def __init__(self, cos_region=ClientRequest.COS_REGION, cos_secret_id=ClientRequest.COS_SECRET_ID,
                 cos_secret_key=ClientRequest.COS_SECRET_KEY):
        self.cos_client = self._create_txy_client(cos_region, cos_secret_id, cos_secret_key)

    def _create_txy_client(self, cos_region, cos_secret_id, cos_secret_key):
        '''
        腾讯云上传client对象
        '''

        try:
            config = CosConfig(Region=cos_region, Secret_id=cos_secret_id,
                               Secret_key=cos_secret_key,
                               )
            client = CosS3Client(config)

            klogger.info('_create_txy_client success')
            return client
        except Exception as e:
            raise Exception('_create_txy_client error:{}'.format(e))

    def cos_upload(self, file_apath: str, cos_key_dir: str, cos_bucket=ClientRequest.COS_BUCKET, retry=2):
        '''

        :param file_apath:
        :param cos_key_dir:
        :param cos_bucket
        :param retry:
        :return:
        '''
        if os.path.isfile(file_apath):
            while retry > 0:
                retry -= 1
                try:
                    cos_key = '{}/'.format(cos_key_dir) + os.path.split(file_apath)[1]
                    resp = self.cos_client.upload_file(
                        Bucket=cos_bucket,
                        LocalFilePath=file_apath,
                        Key=cos_key,
                        PartSize=20,
                        MAXThread=10
                    )
                    etag = resp.get('ETag', '')
                    if etag:
                        klogger.info('cos_upload success')
                        return {'cos_etag': etag, 'cos_key': cos_key}
                except Exception as e:
                    klogger.error('upload file error:{}'.format(e))

        else:
            raise Exception('file not exist:{}'.format(file_apath))

    def cos_object_exists(self, cos_key, cos_bucket=ClientRequest.COS_BUCKET):
        '''
        判断cos文件是否存在
        :param cos_key:
        :param cos_bucket
        :return: bool
        '''
        status = self.cos_client.object_exists(
            Bucket=cos_bucket,
            Key=cos_key
        )
        return status

    def cos_get_object(self, cos_key, save_file, cos_bucket=ClientRequest.COS_BUCKET, retry=2):
        '''
        通过 cos_key 下载文件
        :return: save_file
        '''
        while retry > 0:
            retry -= 0
            try:
                resp = self.cos_client.get_object(
                    Bucket=cos_bucket,
                    Key=cos_key,
                )

                body = resp['Body']
                body.get_stream_to_file(save_file)
                klogger.info('cos_get_object success:{}'.format(save_file))
                return save_file
            except Exception as e:
                klogger.error('cos_get_object error:{}'.format(e))
                raise e

    def cos_get_presigned_download_url(self, cos_key, bucket=ClientRequest.COS_BUCKET, expired=600, params={}, headers={}):
        '''
        生成预签名的url,通过 url 下载文件,url有效期
        :param cos_key:
        :param bucket:
        :param expired: 秒
        :return:
        '''
        url = self.cos_client.get_presigned_download_url(bucket, cos_key, Expired=expired, Params=params,
                                                         Headers=headers)
        return url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
import os
from loguru import logger as klogger

LOG_DIR = os.path.join(os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__)))),
                       'logs')  # 项目根目录/logs


def init_conf_log(log_path: str):
    '''
    初始化disconf日志模块
    :param log_path: 日志输出路径
    :return:
    '''
    if not os.path.exists(log_path):
        os.makedirs(log_path)
    klogger.add(os.path.join(log_path, 'disconf.log'), filter='clife_svc.disconf')


def init_svc_log(log_path: str, log_level='INFO'):
    '''
    初始化服务日志模块
    :param log_path: 日志输出路径
    :param log_level: 日志级别，从低到高依次为 TRACE|DEBUG|INFO|SUCCESS|WARNING|ERROR|CRITICAL
    :return:
    '''
    if not os.path.exists(log_path):
        os.makedirs(log_path)
    klogger.add(
        os.path.join(log_path, '{time:YYYY-MM-DD}.log'),
        # format='{time:YYYY-MM-DD HH:mm:ss} | {level} | {message}',
        level=log_level,
        enqueue=True,
        rotation='500 MB',
        retention='7 days',
        encoding='utf-8'
    )


if __name__ == '__main__':
    print(os.path.dirname(__file__))
    print(os.path.dirname(os.path.dirname(__file__)))
    print(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
    print(os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__)))))



from setuptools import setup

setup(
    name='clife_svc',   # 需要打包的名字,即本模块要发布的名字
    version='0.91',     # 版本
    description='A module for service',   # 简要描述
    packages=['clife_svc', 'clife_svc.errors', 'clife_svc.libs'],
    install_requires=[
        'numpy',
        'loguru',
        'aiohttp',
        'opencv_python',
        'uvicorn',
        'requests',
        'cos_python_sdk_v5',
        'aiofiles',
        'fastapi'
    ],
    author='andy.hu',         # 作者名
    author_email='hlp0@163.com',       # 作者邮件
    url='',       # 项目地址,一般是代码托管的网站
    # requires=['requests','urllib3'], # 依赖包,如果没有,可以不要
    license='MIT'
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
from clife_svc.libs.log import klogger

import requests
import os
import sys

_ENVIRONMENT = int(os.environ.get("ENVIRONMENT", -1))

if _ENVIRONMENT == 0:
    # 开发环境
    # disconf_url = 'http://10.6.14.85:8099/disconf-web/api'
    disconf_url = 'http://disconf.clife.net:8099/disconf-web/api'
    env_name = 'rd'

elif _ENVIRONMENT == 1:
    # 测试环境
    # disconf_url = 'http://10.6.14.85:8099/disconf-web/api'
    disconf_url = 'http://disconf.clife.net:8099/disconf-web/api'
    env_name = 'itest'

elif _ENVIRONMENT == 2:
    # 生产环境
    # disconf_url = 'http://10.186.39.90:8099/disconf-web/api'
    disconf_url = 'http://disconf.clife.net:8099/disconf-web/api'
    env_name = 'res'

else:
    # 本地环境
    disconf_url = 'http://10.6.14.85:8099/disconf-web/api'
    env_name = 'rd'

#_ROOT_DIR = os.path.abspath(os.path.dirname(__file__))
_ROOT_DIR = os.path.abspath(os.path.dirname(sys.argv[0]))
klogger.info('root_path:{}'.format(_ROOT_DIR))

disconf_path = os.path.join(_ROOT_DIR, 'local.properties')  # disconf下载配置文件到根路径


class Disconf:

    '''
    从 disconf 下载配置文件
    '''

    def __init__(self, apps, version, keys: tuple):
        ''' 构造函数
        :param apps: disconf配置app，多个使用逗号分隔
        :param version: 配置版本号
        :param keys: 配置文件名，缺省时只加载ai-commons.properties,tf-serving.properties
        '''

        klogger.info('ENVIRONMENT:{}'.format(_ENVIRONMENT))
        klogger.info('disconf_url:{}'.format(disconf_url))
        klogger.info('env_name:{}'.format(env_name))
        self.conf_url = disconf_url
        self.sess = requests.session()
        self.version = version
        self.apps = apps
        # 默认的配置文件名称
        self.config = 'ai-commons.properties,tf-serving.properties'
        for var in list(keys):
            self.config = self.config + ',' + var

    @staticmethod
    def disconf2dict(disconf_path) -> dict:
        '''
        读取本地disconf文件转dict
        :param disconf_path 本地配置文件路径
        '''

        with open(disconf_path, 'r') as f:
            lines = f.readlines()

        disconf_item = {}
        for line in lines:
            if line:
                line = line.replace('\n', '')
                kv_line = line.split('=', 1)
                if len(kv_line) == 2:
                    k = kv_line[0]
                    v = kv_line[1]
                    if k:
                        disconf_item[k] = v
        return disconf_item

    @staticmethod
    def save_item(item, path):
        '''
        保存配置数据字典至本地文件
        :param item: 配置数据字典
        :param path: 存储文件路径
        :return:
        '''

        with open(path, 'w', encoding='utf-8') as f:
            for k in item:
                f.write(k + '=' + item[k] + '\n')

        klogger.info('save disconf path:{}'.format(path))

    @staticmethod
    def __format_value(value) -> dict:
        '''
        格式化提取配置文件内容，只提取包含'='和key的非注释行，同时对有效行的key和value去除首尾空格
        :param value: 配置文件内容
        :return: 有效的（包含key且非#注释行）配置项字典
        '''
        value = value.replace('↵', '\n')
        # 先根据换行符分割成行列表
        value = value.split('\n')
        # 提取出非注释行，同时包含"="的行
        fields = [v for v in value if '=' in v and not v.startswith('#')]
        item_value = {}
        for field in fields:
            f = field.split('=', 1)
            if len(f) == 2:
                k = f[0]
                v = f[1]  # v值可为空
                if k:  # k不能为空
                    k = k.strip()  # 去除首尾空格
                    if v:
                        v = v.strip()  # 去除首尾空格
                    item_value[k] = v
        return item_value

    def config_file(self, app, version, env, key) -> dict:
        '''
        从配置中心获取配置文件
        :param app: disconf配置app
        :param version: disconf配置版本
        :param env: disconf配置环境
        :param key: disconf配置文件的key（即配置文件名称）
        :return: 配置数据字典
        '''

        url = self.conf_url + '/config/file'
        params = {
            'app': app,
            'version': version,
            'env': env,
            'key': key
        }
        try:
            resp = self.sess.get(url, params=params, timeout=10)
            if resp.status_code == 200:
                try:
                    return self.__format_value(resp.text)
                except Exception as e:
                    klogger.error('error format_value：{}'.format(e))
                    raise e
            else:
                klogger.error('get disconf error,status_code:{},resp.text:{}'.format(resp.status_code, resp.text))
        except Exception as e:
            klogger.error('get disconf error:{}'.format(e))
            raise e

    def set_conf_url_host(self, host: str):
        self.conf_url = 'http://' + host + ':8099/disconf-web/api'

    def get_files(self, app_names: list, version: str, env_name: str, disconf_conf_name: list, local_path: str) -> dict:
        '''
        批量获取disconf配置文件内容并保存至本地
        :param app_names: disconf配置app名称
        :param version: disconf配置版本
        :param env_name: disconf配置环境
        :param disconf_conf_name: 配置文件名称
        :param local_path: 配置数据字典本地存储路径
        :return: 配置数据字典
        '''

        config_items = {}
        for app_name in app_names:
            for conf_key in disconf_conf_name:
                item = self.config_file(app_name, version, env_name, conf_key)
                if item:
                    config_items.update(item)

        self.save_item(config_items, local_path)
        klogger.info('download disconf succcess')
        return config_items

    def get_config_dict(self) ->dict:
        '''
        获取当前项目下的所有配置数据字典
        :return:
        '''

        def split2list(v):
            v = v.replace('，', ',')
            return [a for a in v.split(',') if a]

        try:
            if not isinstance(self.apps, list):
                app_names = split2list(self.apps)

            if not isinstance(self.config, list):
                disconf_conf_name = split2list(self.config)

            klogger.info('start download disconf file')
            disconf_item = self.get_files(app_names, self.version, env_name, disconf_conf_name, disconf_path)

            self.sess.close()
            if not disconf_item:
                klogger.info('error download disconf from disconf_url:{}'.format(disconf_url))
                klogger.info('error disconf_item:{}'.format(disconf_item))
                klogger.info('start load local disconf')
                disconf_item = Disconf.disconf2dict(disconf_path)
            for item in disconf_item:
                klogger.info('disconf_item:{}'.format(item + '=' + disconf_item[item]))

            return disconf_item
        except Exception as e:
            klogger.error('error download disconf:{}'.format(e))
            raise e
